using System;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Drawing.Printing;

namespace TYWinforms.Day11 {
   public class Exercise1 : Form {
      private TreeView tvFiles = new TreeView();
      private Label lblInfo = new Label();
      private MainMenu mnuMain = new MainMenu();
      private String strList = "";
      private StringReader objPrintReader;

      public Exercise1() {
         tvFiles.Dock = DockStyle.Left;
         tvFiles.Width = 200;
         tvFiles.AfterSelect += new TreeViewEventHandler(this.DisplayInfo);
         tvFiles.BeforeExpand += new TreeViewCancelEventHandler(this.ExpandThread);

         lblInfo.Size = new Size(150,150);
         lblInfo.Location = new Point(210,10);
         lblInfo.Text = "Select a file or\ndirectory";

         PopulateList("c:\\", tvFiles.Nodes.Add("c:\\"));

         MenuItem miFile = new MenuItem("File");
         MenuItem miPrint = new MenuItem("Print");
         miPrint.Click += new EventHandler(this.PrintClicked);

         mnuMain.MenuItems.Add(miFile);
         miFile.MenuItems.Add(miPrint);

         this.Menu = mnuMain;
         this.Text = "Exercise 1";
         this.Size = new Size(800,600);
         this.Controls.Add(tvFiles);
         this.Controls.Add(lblInfo);         
      }

      private void PopulateList(String strPath, TreeNode currNode) {
         DirectoryInfo dir = new DirectoryInfo(strPath);
         TreeNode nodeSubDir;

         foreach (DirectoryInfo d in dir.GetDirectories()) {
            nodeSubDir = currNode.Nodes.Add(d.Name);
            nodeSubDir.Nodes.Add("");
         }

         foreach (FileInfo f in dir.GetFiles("*.*")) {
            currNode.Nodes.Add(f.Name);
         }
      }

      private void ExpandThread(Object Sender, TreeViewCancelEventArgs e) {
         if (e.Node.Nodes[0].Text == "") {
            TreeNode tempNode = e.Node;
            String strFullName = e.Node.Text;

            while (tempNode.Parent != null) {
               strFullName = tempNode.Parent.Text + "\\" + strFullName;
               tempNode = tempNode.Parent;
            }

            e.Node.Nodes[0].Remove();
            PopulateList(strFullName, e.Node); 
         }
      }

      private void DisplayInfo(Object Sender, TreeViewEventArgs e) {
         TreeNode tempNode = e.Node;
         String strFullName = tempNode.Text;

         while (tempNode.Parent != null) {
            strFullName = tempNode.Parent.Text + "\\" + strFullName;
            tempNode = tempNode.Parent;
         }

         if (File.Exists(strFullName)) {
            FileInfo obj = new FileInfo(strFullName);
            lblInfo.Text = "Name: " + obj.Name;
            lblInfo.Text += "\nSize: " + obj.Length;
            lblInfo.Text += "\nAccessed: " + obj.LastAccessTime.ToString();
         } else {
            DirectoryInfo obj = new DirectoryInfo(strFullName);
            lblInfo.Text = "Name: " + obj.Name;
            lblInfo.Text += "\nAttributes: " + obj.Attributes.ToString();
            lblInfo.Text += "\nAccessed: " + obj.LastAccessTime.ToString();
         }
      }

      private void PrintClicked(Object Sender, EventArgs e) {
         //Assemble string of nodes
         strList = "";
         TreeNode nodeTemp = tvFiles.TopNode;

         for (int i = 0; i < tvFiles.GetNodeCount(true); i++) {
            /*pad the string with spaces for easier
              reading */
            strList += "".PadRight(GetLevel(nodeTemp)*2) + nodeTemp.Text + "\n";

            nodeTemp = nodeTemp.NextVisibleNode;
            if (nodeTemp == null) break;

            nodeTemp.EnsureVisible();
         }

         PrintDocument pd = new PrintDocument();
         pd.PrintPage += new PrintPageEventHandler(this.pd_PrintPage);
         objPrintReader = new StringReader(strList);
         pd.Print();
      }

      /* Returns an integer indicating how many levels deep 
         the specified node is in the treeview */
      private int GetLevel(TreeNode objNode) {
         int intLevel = 0;
         TreeNode nodeTemp = objNode;
 
         while (nodeTemp.Parent != null) {
            intLevel++;
            nodeTemp = nodeTemp.Parent;
         }

         return intLevel;
      }

      private void pd_PrintPage(Object Sender, PrintPageEventArgs e) {
         Font fntPrint = this.Font;
         int count = 0; 
         float yPos = 0;
         float lpp = e.MarginBounds.Height / fntPrint.GetHeight(e.Graphics);
         float fltTopMargin = e.MarginBounds.Top;
         float fltLeftMargin = e.MarginBounds.Left;
         String strLine = null;

         while (count < lpp && ((strLine = objPrintReader.ReadLine()) != null)) {
            yPos = fltTopMargin + (count * fntPrint.GetHeight(e.Graphics));

            e.Graphics.DrawString(strLine, fntPrint, Brushes.Black, fltLeftMargin, yPos, new StringFormat());

            count++;
         }

         if (strLine != null) {
            e.HasMorePages = true;
         } else {
            e.HasMorePages = false;
         }
      }

      public static void Main() {
         Application.Run(new Exercise1());
      }
   }
}