using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace TYWinforms.Day13 {
   public class Exercise : Form {
      private ToolBar tlbStandard = new ToolBar();
      private string strFunction;
      private Point ptStartClick;
      private Point ptCurrentPos;
      private bool blnDrag = false;
      private bool blnFill = false;
      private SolidBrush objBrush;
      private Pen objPen = new Pen(Color.Black);
      private Rectangle rectSelect;
      private Color currColor = Color.Black;
      private Bitmap imgWorking;
      private Graphics objG;

      public Exercise() {
         imgWorking = new Bitmap(800,600,PixelFormat.Format32bppArgb);
         objG = Graphics.FromImage(imgWorking);
         objG.FillRectangle(new SolidBrush(Color.White), new Rectangle(0,0,800,600));

         ToolBarButton tbbPen = new ToolBarButton();
         tbbPen.Text = "Pencil";

         ToolBarButton tbbRect = new ToolBarButton();
         tbbRect.Text = "Rectangle";

         ToolBarButton tbbErase = new ToolBarButton();
         tbbErase.Text = "Erase";

         ToolBarButton tbbColor = new ToolBarButton();
         tbbColor.Text = "Color...";

         tlbStandard.Buttons.Add(tbbPen);
         tlbStandard.Buttons.Add(tbbRect);
         tlbStandard.Buttons.Add(tbbErase);
         tlbStandard.Buttons.Add(tbbColor);
         tlbStandard.Appearance = ToolBarAppearance.Flat;

         tlbStandard.ButtonClick += new ToolBarButtonClickEventHandler(this.MyToolBarHandler);

         this.BackgroundImage = imgWorking;
         this.Text = "Exercise";
         this.Size = new Size(800,600);
         this.Controls.Add(tlbStandard);
      }

      private void MyToolBarHandler(Object Sender, ToolBarButtonClickEventArgs e) {
         if (e.Button.Text == "Color...") {
            ColorDialog dlgColor = new ColorDialog();

            if (dlgColor.ShowDialog() == DialogResult.OK) {
               currColor = dlgColor.Color;
            }
         } else {
            strFunction = e.Button.Text;
            tlbStandard.Buttons[0].Pushed = false;
            tlbStandard.Buttons[1].Pushed = false;
            tlbStandard.Buttons[2].Pushed = false;
            e.Button.Pushed = true;
         }
     
      }

      protected override void OnMouseDown(MouseEventArgs e) {
         blnDrag = true;
         ptStartClick = new Point(e.X, e.Y);
         ptCurrentPos = ptStartClick;
      }

      protected override void OnMouseUp(MouseEventArgs e) {
         blnDrag = false;
         blnFill = true;

         DrawIt(objG);
      }

      protected override void OnMouseMove(MouseEventArgs e) {
         ptCurrentPos = new Point(e.X, e.Y);

         if (blnDrag | blnFill) {
            DrawIt(objG);
         }
      }

      private void DrawIt(Graphics objGraphics) {
         switch(strFunction) {
            case "Pencil":
               objPen.Color = currColor;
               objPen.Width = 2;

               if (blnDrag) {
                  objGraphics.DrawLine(objPen, ptStartClick, ptCurrentPos);
                  ptStartClick = ptCurrentPos;
                  this.Invalidate();
               }
               break;
            case "Rectangle":
               if (blnDrag) {
                  if (ptCurrentPos.X < ptStartClick.X & ptCurrentPos.Y < ptStartClick.Y) {
                     rectSelect = new Rectangle(ptCurrentPos.X, ptCurrentPos.Y, ptStartClick.X - ptCurrentPos.X, ptStartClick.Y - ptCurrentPos.Y);
                  } else if (ptCurrentPos.X < ptStartClick.X & ptCurrentPos.Y > ptStartClick.Y) {
                     rectSelect = new Rectangle(ptCurrentPos.X, ptStartClick.Y, ptStartClick.X - ptCurrentPos.X, ptCurrentPos.Y - ptStartClick.Y);
                  } else if (ptCurrentPos.X > ptStartClick.X & ptCurrentPos.Y < ptStartClick.Y) {
                     rectSelect = new Rectangle(ptStartClick.X, ptCurrentPos.Y, ptCurrentPos.X - ptStartClick.X, ptStartClick.Y - ptCurrentPos.Y);
                  } else {
                     rectSelect = new Rectangle(ptStartClick.X, ptStartClick.Y, ptCurrentPos.X - ptStartClick.X, ptCurrentPos.Y - ptStartClick.Y);
                  }
               } 

               if (blnFill) {
                  objBrush = new SolidBrush(currColor);

                  objGraphics.FillRectangle(objBrush, rectSelect);
                  this.Invalidate();
                  blnFill = false;
               }
               break;
            case "Erase":
               objPen.Color = Color.White;
               objPen.Width = 5;

               if (blnDrag) {
                  objGraphics.DrawLine(objPen, ptStartClick, ptCurrentPos);
                  ptStartClick = ptCurrentPos;
                  this.Invalidate();
               }
               break;
         }
      }

      public static void Main() {
         Application.Run(new Exercise());
      }
   }
}