Imports System.ServiceProcess
Imports System.IO

namespace TYWinforms.Day16

   Public Class Day16Service : Inherits ServiceBase
      private objWatcher As New FileSystemWatcher()
      private objLog as New System.Diagnostics.EventLog()

      Public Sub New()
         If Not EventLog.SourceExists("TYWinforms Day 16 Service") Then
            EventLog.CreateEventSource("TYWinforms Day 16 Service", "MyNewLog")
         End If

         objLog.Source = "TYWinforms Day 16 Service"

         objWatcher.Path = "c:\"
         objWatcher.IncludeSubdirectories = True
         objWatcher.NotifyFilter = (NotifyFilters.FileName Or NotifyFilters.DirectoryName Or NotifyFilters.LastWrite)
         objWatcher.EnableRaisingEvents = True

         AddHandler objWatcher.Changed, new FileSystemEventHandler(AddressOf Me.FileChanged)
         AddHandler objWatcher.Deleted, new FileSystemEventHandler(AddressOf Me.FileDeleted)
         AddHandler objWatcher.Created, new FileSystemEventHandler(AddressOf Me.FileCreated)
         AddHandler objWatcher.Renamed, new RenamedEventHandler(AddressOf Me.FileRenamed)
      
         Me.ServiceName = "TYWinforms Day 16 Service"
      End Sub

      Shared Sub Main()
         ServiceBase.Run(New Day16Service())
      End Sub

      Protected Overrides Sub OnStart(ByVal args() As String)
         objLog.WriteEntry("TYWinforms Day 16 Service is starting")
      End Sub

      Protected Sub FileChanged(Sender As Object, e As FileSystemEventArgs) 
         objLog.WriteEntry(e.Name & " " & e.ChangeType.ToString())
      End Sub

      Public Sub FileCreated(Sender As Object, e As FileSystemEventArgs)
         objLog.WriteEntry(e.Name & " created at: " & e.FullPath)
      End Sub

      Public Sub FileDeleted(Sender As Object, e As FileSystemEventArgs)
         objLog.WriteEntry(e.Name & " deleted from: " & e.FullPath)
      End Sub

      Public Sub FileRenamed(Sender As Object, e As RenamedEventArgs)
         objLog.WriteEntry(e.OldName & " renamed to: " & e.Name)
      End Sub
   End Class
end namespace