using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Printing;
using System.ComponentModel;

namespace TYWinforms.P1 {
   public class NetWord : Form {
      private RichTextBox rtbDocument = new RichTextBox();
      private ContextMenu cmnuDocument = new ContextMenu();
      private StatusBar sbarMain = new StatusBar();
      private StatusBarPanel spnlLine = new StatusBarPanel();

      private MainMenu mnuMain = new MainMenu();
      private MenuItem mniFile = new MenuItem("File");
      private MenuItem mniOpen = new MenuItem("Open...");
      private MenuItem mniSave = new MenuItem("Save");
      private MenuItem mniPageSetup = new MenuItem("Page Setup...");
      private MenuItem mniPrintPreview = new MenuItem("Print Preview");
      private MenuItem mniPrint = new MenuItem("Print...");
      private MenuItem mniExit = new MenuItem("Exit");

      private MenuItem mniEdit = new MenuItem("Edit");
      private MenuItem mniUndo = new MenuItem("Undo");
      private MenuItem mniCut = new MenuItem("Cut");
      private MenuItem mniCopy = new MenuItem("Copy");
      private MenuItem mniPaste = new MenuItem("Paste");
      private MenuItem mniFind = new MenuItem("Find...");

      private MenuItem mniFormat = new MenuItem("Format");
      private MenuItem mniFont = new MenuItem("Font...");

      private Font fntCurrent = new Font("Times New Roman", 10);
      private Color fntColor = Color.Black;
      private FontDialog dlgFont = new FontDialog();
      private OpenFileDialog dlgOpen = new OpenFileDialog();
      private SaveFileDialog dlgSave = new SaveFileDialog();
      private PrintPreviewDialog dlgPrintPreview = new PrintPreviewDialog();
      private PageSetupDialog dlgPageSetup = new PageSetupDialog();
      private PrintDialog dlgPrint = new PrintDialog();

      private PageSettings objPageSettings = new PageSettings();
      private PrinterSettings objPrintSettings = new PrinterSettings();

      public NetWord() {
         rtbDocument.Dock = DockStyle.Fill;
         rtbDocument.Font = fntCurrent;
         rtbDocument.HideSelection = false;
         rtbDocument.AcceptsTab = true;
         rtbDocument.SelectionChanged += new EventHandler(this.UpdateStatus);

         sbarMain.ShowPanels = true;
         sbarMain.Panels.Add(spnlLine);
         spnlLine.AutoSize = StatusBarPanelAutoSize.Spring;
         spnlLine.Alignment = HorizontalAlignment.Right;

         mniOpen.Click += new EventHandler(this.FileClicked);
         mniSave.Click += new EventHandler(this.FileClicked);
         mniPageSetup.Click += new EventHandler(this.FileClicked);
         mniPrintPreview.Click += new EventHandler(this.FileClicked);
         mniPrint.Click += new EventHandler(this.FileClicked);
         mniExit.Click += new EventHandler(this.FileClicked);

         mniUndo.Click += new EventHandler(this.EditClicked);
         mniCut.Click += new EventHandler(this.EditClicked);
         mniCopy.Click += new EventHandler(this.EditClicked);
         mniPaste.Click += new EventHandler(this.EditClicked);
         mniFind.Click += new EventHandler(this.EditClicked);

         mniFont.Click += new EventHandler(this.FormatClicked);

         mnuMain.MenuItems.Add(mniFile);
         mnuMain.MenuItems.Add(mniEdit);
         mnuMain.MenuItems.Add(mniFormat);
         
         mniOpen.Shortcut = Shortcut.CtrlO;
         mniSave.Shortcut = Shortcut.CtrlS;
         mniPrint.Shortcut = Shortcut.CtrlP;
         mniUndo.Shortcut = Shortcut.CtrlZ;
         mniCut.Shortcut = Shortcut.CtrlX;
         mniCopy.Shortcut = Shortcut.CtrlC;
         mniPaste.Shortcut = Shortcut.CtrlV;
         mniFind.Shortcut = Shortcut.CtrlF;

         mniOpen.ShowShortcut = true;
         mniSave.ShowShortcut = true;
         mniPrint.ShowShortcut = true;
         mniUndo.ShowShortcut = true;
         mniCut.ShowShortcut = true;
         mniCopy.ShowShortcut = true;
         mniPaste.ShowShortcut = true;
         mniFind.ShowShortcut = true;

         mniFile.MenuItems.Add(mniOpen);
         mniFile.MenuItems.Add(mniSave);
         mniFile.MenuItems.Add("-");
         mniFile.MenuItems.Add(mniPageSetup);
         mniFile.MenuItems.Add(mniPrintPreview);
         mniFile.MenuItems.Add(mniPrint);
         mniFile.MenuItems.Add("-");
         mniFile.MenuItems.Add(mniExit);

         mniEdit.MenuItems.Add(mniUndo);
         mniEdit.MenuItems.Add("-");
         mniEdit.MenuItems.Add(mniCut);
         mniEdit.MenuItems.Add(mniCopy);
         mniEdit.MenuItems.Add(mniPaste);
         mniEdit.MenuItems.Add("-");
         mniEdit.MenuItems.Add(mniFind);

         mniFormat.MenuItems.Add(mniFont);

         cmnuDocument.MenuItems.Add(mniCut.CloneMenu());
         cmnuDocument.MenuItems.Add(mniCopy.CloneMenu());
         cmnuDocument.MenuItems.Add(mniPaste.CloneMenu());
         cmnuDocument.MenuItems.Add("-");
         cmnuDocument.MenuItems.Add(mniFont.CloneMenu());
         cmnuDocument.Popup += new EventHandler(this.HandleContext);
         rtbDocument.ContextMenu = cmnuDocument;

         this.Text = "NetWord v.1";
         this.Name = "NetWord";
         this.Size = new Size(800,600);
         this.Menu = mnuMain;
         this.Closing += new CancelEventHandler(this.DocumentClosing);         

         this.Controls.Add(rtbDocument);
         this.Controls.Add(sbarMain);
      }

      private void UpdateStatus(Object Sender, EventArgs e) {
         String text = rtbDocument.Text;

         int intLine = rtbDocument.GetLineFromCharIndex(rtbDocument.SelectionStart) + 1;

         int i = rtbDocument.SelectionStart;
         while (rtbDocument.GetPositionFromCharIndex(i).X > 1) {
            --i;
         }
         String strChar = (rtbDocument.SelectionStart - i + 1).ToString();

         spnlLine.Text = "Line " + intLine.ToString() + " Char " + strChar;
      }

      private void HandleContext(Object Sender, EventArgs e) {
         if (rtbDocument.SelectionLength == 0) {
            cmnuDocument.MenuItems[0].Enabled = false;
            cmnuDocument.MenuItems[1].Enabled = false;
         } else {
            cmnuDocument.MenuItems[0].Enabled = true;
            cmnuDocument.MenuItems[1].Enabled = true;
         }
      }

      private void FileClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         switch (mniTemp.Text) {
            case "Open...":
               if (dlgOpen.ShowDialog() == DialogResult.OK) {
                  //open file
               }
               break;
            case "Save":
               if (dlgSave.ShowDialog() == DialogResult.OK) {
                  //save file
               }
               break;
            case "Page Setup...":
               dlgPageSetup.PageSettings = objPageSettings;
               dlgPageSetup.ShowDialog();
               break;
            case "Print Preview":
               dlgPrintPreview.Document = new PrintDocument();
               dlgPrintPreview.ShowDialog();
               break;
            case "Print...":
               dlgPrint.PrinterSettings = new PrinterSettings();
               if (dlgPrint.ShowDialog() == DialogResult.OK) {
                  //print
               }
               break;
            case "Exit":
               this.Close();
               break;
         }
      }

      private void EditClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         switch (mniTemp.Text) {
            case "Undo":
               rtbDocument.Undo();
               break;
            case "Cut":
               if (rtbDocument.SelectedRtf != "") {
                  rtbDocument.Cut();
               }
               break;
            case "Copy":
               if (rtbDocument.SelectedRtf != "") {
                  rtbDocument.Copy();
               }
               break;
            case "Paste":
               rtbDocument.Paste();
               break;
            case "Find...":
               FindDialog dlgFind = new FindDialog(rtbDocument);
               dlgFind.Show();
               break;
         }
      }

      private void FormatClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         switch (mniTemp.Text) {
            case "Font...":
               dlgFont.ShowColor = true;
               if (dlgFont.ShowDialog() == DialogResult.OK) {
                  fntCurrent = dlgFont.Font;
                  fntColor = dlgFont.Color;
                  rtbDocument.SelectionFont = fntCurrent;
                  rtbDocument.SelectionColor = fntColor;
               }
               break;
         }
      }

      private void DocumentClosing(Object Sender, CancelEventArgs e) {
         if (rtbDocument.Modified) {
            DialogResult dr = MessageBox.Show("Do you want to save the changes?", this.Name, MessageBoxButtons.YesNoCancel);

            if (dr == DialogResult.Yes) {
               MessageBox.Show("Too bad!", this.Name);
            } else if (dr == DialogResult.Cancel) {
               e.Cancel = true;
            }
         }
      }

      public static void Main() {
         Application.Run(new NetWord());
      }
   }
}