using System;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Drawing.Printing;
using System.ComponentModel;

namespace TYWinforms.P3 {
   public class NetWord : Form {
      private MainMenu mnuMain = new MainMenu();
      private MenuItem mniFile = new MenuItem("File");
      private MenuItem mniNew = new MenuItem("New");
      private MenuItem mniOpen = new MenuItem("Open...");
      private MenuItem mniExit = new MenuItem("Exit");
      private MenuItem mniWindow = new MenuItem("Window");
      private MenuItem mniWinC = new MenuItem("Cascade");
      private MenuItem mniWinTH = new MenuItem("Tile Horizontal");
      private MenuItem mniWinTV = new MenuItem("Tile Vertical");
 
      private int intCounter = 0;

      public NetWord() {
         mniNew.Click += new EventHandler(this.FileClicked);
         mniOpen.Click += new EventHandler(this.FileClicked);
         mniExit.Click += new EventHandler(this.FileClicked);
         mniWinC.Click += new EventHandler(this.WindowClicked);
         mniWinTH.Click += new EventHandler(this.WindowClicked);
         mniWinTV.Click += new EventHandler(this.WindowClicked);

         mnuMain.MenuItems.Add(mniFile);
         mnuMain.MenuItems.Add(mniWindow);
         
         mniFile.MergeType = MenuMerge.MergeItems;
         mniFile.MenuItems.Add(mniNew);
         mniFile.MenuItems.Add(mniOpen);
         mniFile.MenuItems.Add("-");
         mniFile.MenuItems.Add(mniExit);

         mniWindow.MdiList = true;
         mniWindow.MenuItems.Add(mniWinC);
         mniWindow.MenuItems.Add(mniWinTH);
         mniWindow.MenuItems.Add(mniWinTV);

         mniFile.MergeOrder = 1;
         mniWindow.MergeOrder = 5;
         mniNew.MergeOrder = 10;
         mniOpen.MergeOrder = 20;
         mniExit.MergeOrder = 99;
         mniFile.MenuItems[2].MergeOrder = 98;

         mniOpen.Shortcut = Shortcut.CtrlO;

         this.IsMdiContainer = true;
         this.Text = "NetWord v.3";
         this.Name = "NetWord";
         this.Size = new Size(800,900);
         this.Menu = mnuMain;
      }

      private void FileClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;
         Document doc;

         switch (mniTemp.Text) {
            case "New":
               intCounter++; 

               doc = new Document("Document " + intCounter);
               doc.MdiParent = this;
               doc.Show();
               break;
            case "Open...":
               OpenFileDialog dlgOpen = new OpenFileDialog();
               dlgOpen.Filter = "All files (*.*)|*.*";

               if (dlgOpen.ShowDialog() == DialogResult.OK) {
                  intCounter++;

                  doc = new Document(dlgOpen.FileName);
                  doc.MdiParent = this;
                  FileInfo filTemp = new FileInfo(dlgOpen.FileName);
                  if (filTemp.Extension == ".rtf") {
                     ((RichTextBox)doc.arrPages[0]).LoadFile(dlgOpen.FileName, RichTextBoxStreamType.RichText);
                  } else {
                     ((RichTextBox)doc.arrPages[0]).LoadFile(dlgOpen.FileName, RichTextBoxStreamType.PlainText);
                  }

                  ((RichTextBox)doc.arrPages[0]).Modified = false;
                  doc.Show();
               }
               break;
            case "Exit":
               this.Close();
               break;
         }
      }

      private void WindowClicked(Object Sender, EventArgs e) {
         MenuItem miTemp = (MenuItem)Sender;
  
         switch (miTemp.Text) {
            case "Cascade":
               this.LayoutMdi(MdiLayout.Cascade);
               break;
            case "Tile Horizontal":
               this.LayoutMdi(MdiLayout.TileHorizontal);
               break;
            case "Tile Vertical":
               this.LayoutMdi(MdiLayout.TileVertical);
               break;
         }
      }

      public static void Main() {
         Application.Run(new NetWord());
      }
   }
}