using System;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Printing;

namespace TYWinforms.P3 {
   public class Page : RichTextBox {
      public int PageNumber;
      private int intCharIndex;
      private int intLineIndex;
      public event EventHandler Filled;

      private PageSettings objPageSettings;

      public int CharIndex {
         get { return intCharIndex; }
      }

      public int LineIndex {
         get { return intLineIndex; }
      }

      public Page(Form frmParent, PageSettings objPageSettings, int intPageNumber) {
         intCharIndex = 1;
         intLineIndex = 1;
         PageNumber = intPageNumber;
         int intWidth = (int)(.96 * (objPageSettings.Bounds.Width - (objPageSettings.Margins.Left + objPageSettings.Margins.Right)));
         int intHeight = (int)(.96 * (objPageSettings.Bounds.Height - (objPageSettings.Margins.Top + objPageSettings.Margins.Bottom)));

         this.Parent = frmParent;
         this.ScrollBars = RichTextBoxScrollBars.None;
         this.Font = frmParent.Font;
         this.objPageSettings = objPageSettings;
         this.RightMargin = objPageSettings.Bounds.Width - (int)(.96 * (objPageSettings.Margins.Left + objPageSettings.Margins.Right));
         this.Size = new Size(intWidth,intHeight);
      }

      public void SetFont(Font fnt, Color clr) {
         this.SelectionFont = fnt;
         this.SelectionColor = clr;
      }

      protected virtual void OnFilled(EventArgs e) {
         Filled(this, e);
      }

      public void UpdatePageSettings(PageSettings objPageSettings) {
         this.objPageSettings = objPageSettings;

         this.RightMargin = objPageSettings.Bounds.Width - (int)(.96 * (objPageSettings.Margins.Left + objPageSettings.Margins.Right));
      }

      protected override void OnSelectionChanged(EventArgs e) {
         intLineIndex = this.GetLineFromCharIndex(this.SelectionStart) + 1;

         int i = this.SelectionStart;
         while (this.GetPositionFromCharIndex(i).X > 1) {
            --i;
         }

         intCharIndex = SelectionStart - i + 1;

         if (this.GetPositionFromCharIndex(this.SelectionStart).Y + (this.SelectionFont.GetHeight()*2) >= this.ClientSize.Height) {
            OnFilled(new EventArgs());
         }

         base.OnSelectionChanged(e);
      }
   }
}